'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.clear = exports.apiDestroy = exports.apiEdit = exports.apiShow = exports.apiIndex = exports.apiUpdate = exports.apiCreate = exports.apiNew = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _constants = require('./constants');

var _reduxApiMiddleware = require('redux-api-middleware');

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Builds a request type action.
 */
var apiRequest = function apiRequest(config) {
  return _defineProperty({}, _constants.API, config);
};

/**
 * Build action for new request type action.
 */
var apiNew = function apiNew() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return apiRequest(_extends({}, opts, {
    types: [_constants.ACTIONS.API_NEW_START, _constants.ACTIONS.API_NEW_SUCCESS, _constants.ACTIONS.API_NEW_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.NEW
  }));
};

/**
 * Build action for create request type action.
 */
var apiCreate = function apiCreate() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return apiRequest(_extends({}, opts, {
    method: 'POST',
    types: [_constants.ACTIONS.API_CREATE_START, _constants.ACTIONS.API_CREATE_SUCCESS, _constants.ACTIONS.API_CREATE_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.CREATE
  }));
};

/**
 * Build action for update request type action.
 */
var apiUpdate = function apiUpdate() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return apiRequest(_extends({}, opts, {
    method: 'PUT',
    types: [_constants.ACTIONS.API_UPDATE_START, _constants.ACTIONS.API_UPDATE_SUCCESS, _constants.ACTIONS.API_UPDATE_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.UPDATE
  }));
};

/**
 * Build action for index request type action.
 */
var apiIndex = function apiIndex() {
  var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var _ref2$additive = _ref2.additive,
      additive = _ref2$additive === undefined ? false : _ref2$additive,
      opts = _objectWithoutProperties(_ref2, ['additive']);

  return apiRequest(_extends({}, opts, {
    types: [_constants.ACTIONS.API_INDEX_START, {
      type: _constants.ACTIONS.API_INDEX_SUCCESS,
      payload: function payload(_action, _state, res) {
        var headers = res.headers;

        return (0, _reduxApiMiddleware.getJSON)(res).then(function (json) {
          return {
            records: json,
            pagination: {
              total: parseInt(headers.get('X-Total')),
              perPage: parseInt(headers.get('X-Per-Page')),
              page: parseInt(headers.get('X-Page'))
            }
          };
        });
      },
      meta: {
        additive: additive
      }
    }, _constants.ACTIONS.API_INDEX_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.INDEX
  }));
};

/**
 * Build action for show request type action.
 */
var apiShow = function apiShow() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return apiRequest(_extends({}, opts, {
    types: [_constants.ACTIONS.API_SHOW_START, {
      type: _constants.ACTIONS.API_SHOW_SUCCESS,
      payload: function payload(_action, _state, res) {
        var headers = res.headers;

        return (0, _reduxApiMiddleware.getJSON)(res).then(function (json) {
          return {
            record: json,
            headers: {
              'Etag': headers.get('Etag'),
              'Last-Modified': headers.get('Last-Modified')
            }
          };
        });
      }
    }, _constants.ACTIONS.API_SHOW_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.SHOW
  }));
};

/**
 * Build action for edit request type action.
 */
var apiEdit = function apiEdit() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return apiRequest(_extends({}, opts, {
    types: [_constants.ACTIONS.API_EDIT_START, _constants.ACTIONS.API_EDIT_SUCCESS, _constants.ACTIONS.API_EDIT_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.EDIT
  }));
};

/**
 * Build action for destroy request type action.
 */
var apiDestroy = function apiDestroy() {
  var opts = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return apiRequest(_extends({}, opts, {
    method: 'DELETE',
    types: [_constants.ACTIONS.API_DESTROY_START, _constants.ACTIONS.API_DESTROY_SUCCESS, _constants.ACTIONS.API_DESTROY_FAILURE],
    apiAction: _constants.BACKEND_ACTIONS.DESTROY
  }));
};

/**
 * Build action to clear api action state.
 */
var clear = function clear(apiAction, apiResource) {
  return {
    type: _constants.ACTIONS.CLEAR,
    payload: {
      apiAction: apiAction,
      apiResource: apiResource
    }
  };
};

exports.apiNew = apiNew;
exports.apiCreate = apiCreate;
exports.apiUpdate = apiUpdate;
exports.apiIndex = apiIndex;
exports.apiShow = apiShow;
exports.apiEdit = apiEdit;
exports.apiDestroy = apiDestroy;
exports.clear = clear;