'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _reduxApiMiddleware = require('redux-api-middleware');

var _constants = require('./constants');

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

String.prototype.capitalize = function () {
  return this.charAt(0).toUpperCase() + this.slice(1);
};

/**
 * Generic middleware used to trigger RSAA actions.
 */
var middleware = function middleware() {
  var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  return function (_store) {
    return function (next) {
      return function (action) {
        if (!action.hasOwnProperty(_constants.API)) return next(action);

        var _action$API = action[_constants.API],
            path = _action$API.path,
            _action$API$types = _action$API.types,
            types = _action$API$types === undefined ? [] : _action$API$types,
            apiAction = _action$API.apiAction,
            apiResource = _action$API.apiResource,
            endpoint = _action$API.endpoint,
            _action$API$body = _action$API.body,
            body = _action$API$body === undefined ? {} : _action$API$body,
            _action$API$method = _action$API.method,
            method = _action$API$method === undefined ? 'GET' : _action$API$method,
            _action$API$headers = _action$API.headers,
            headers = _action$API$headers === undefined ? {} : _action$API$headers,
            _action$API$authorize = _action$API.authorize,
            authorize = _action$API$authorize === undefined ? true : _action$API$authorize;

        // Set default headers.

        headers = _extends({
          'Accept': 'application/json',
          'Content-Type': 'application/json'
        }, headers);

        if (authorize && config.session && config.auth) {
          var _extends2;

          var _config$session = config.session,
              token = _config$session.token,
              login = _config$session.login,
              _config$session$user = _config$session.user,
              user = _config$session$user === undefined ? 'user' : _config$session$user;
          var _config$auth = config.auth,
              storage = _config$auth.storage,
              encode = _config$auth.encode,
              decode = _config$auth.decode;


          var tokenStored = storage.getItem(encode(token));
          var loginStored = storage.getItem(encode(login));

          headers = _extends({}, headers, (_extends2 = {}, _defineProperty(_extends2, 'X-' + user.capitalize() + '-Token', decode(tokenStored)), _defineProperty(_extends2, 'X-' + user.capitalize() + '-Email', decode(loginStored)), _extends2));
        }

        var _types = _slicedToArray(types, 3),
            _types$ = _types[0],
            requestType = _types$ === undefined ? '' : _types$,
            _types$2 = _types[1],
            successType = _types$2 === undefined ? '' : _types$2,
            _types$3 = _types[2],
            failureType = _types$3 === undefined ? '' : _types$3;

        if (typeof requestType === 'string') {
          requestType = {
            type: requestType,
            meta: {
              apiAction: apiAction,
              apiResource: apiResource
            }
          };
        } else {
          requestType = _extends({}, requestType, {
            meta: _extends({}, requestType.meta, {
              apiAction: apiAction,
              apiResource: apiResource
            })
          });
        }

        if (typeof successType === 'string') {
          successType = {
            type: successType,
            meta: {
              apiAction: apiAction,
              apiResource: apiResource
            }
          };
        } else {
          successType = _extends({}, successType, {
            meta: _extends({}, successType.meta, {
              apiAction: apiAction,
              apiResource: apiResource
            })
          });
        }

        if (typeof failureType === 'string') {
          failureType = {
            type: failureType,
            meta: function meta(_action, _state, res) {
              return {
                status: {
                  code: res.status
                },
                apiAction: apiAction,
                apiResource: apiResource
              };
            }
          };
        } else {
          failureType = _extends({}, failureType, {
            meta: function meta(_action, _state, res) {
              return _extends({
                status: {
                  code: res.status
                },
                apiAction: apiAction,
                apiResource: apiResource
              }, failureType.meta);
            }
          });
        }

        // Trigger redux api middleware action.
        return next(_defineProperty({}, _reduxApiMiddleware.RSAA, {
          endpoint: endpoint || config.baseUrl + '/' + path,
          body: method === 'GET' ? null : JSON.stringify(body),
          types: [requestType, successType, failureType],
          headers: headers,
          method: method
        }));
      };
    };
  };
};

exports.default = middleware;